import EventEmitter from "node:events"
import type { PlayerBroadcast } from "../shared/broadcast"
import { script } from "../shared/script"
import type { SceneDefinition } from "../shared/script/types"
import { sceneTypesById } from "./scene-types"

interface Events {
    "player-broadcast": [PlayerBroadcast]
}

export interface CurrentScene<Type extends keyof typeof sceneTypesById> {
    id: string
    definition: SceneDefinition & { type: Type }
    state: ReturnType<typeof sceneTypesById[Type]["createState"]>
}

export class Game {
    // @ts-expect-error
    private currentScene: CurrentScene
    public eventBus = new EventEmitter<Events>({ captureRejections: false })

    constructor() {
        this.switchScene("pre-start")
    }

    getConnectionPlayerBroadcasts(): PlayerBroadcast[] {
        const events: PlayerBroadcast[] = []
        events.push({ type: "scene-changed", sceneId: this.currentScene.id })
        events.push(...this.currentScene.state.getConnectionEvents())

        return events
    }

    switchScene(sceneId: string) {
        const definition = script.scenesById.get(sceneId)
        if (definition === undefined) throw new Error(`Unknown scene: ${sceneId}`)
        const type = sceneTypesById[definition.type]
        this.currentScene = {
            id: sceneId,
            definition,
            state: type.createState(this, definition as any)
        }
        this.eventBus.emit("player-broadcast", { type: "scene-changed", sceneId: sceneId })
    }

    withSceneState<Type extends keyof typeof sceneTypesById>(type: Type, block: (state: ReturnType<typeof sceneTypesById[Type]["createState"]>) => void) {
        if (this.currentScene.definition.type === type) {
            block(this.currentScene.state)
        }
    }
}

export const game = new Game()