import { Temporal } from "temporal-polyfill"
import type { Exact } from "type-fest"
import { getSuggestedInteractionFromDefinition, getSuggestedInteractionId } from "../scene-types/interaction"
import type { IdMap } from "../util"

export interface Script {
    scenesById: Map<string, SceneDefinition>
}

export interface SceneDefinitionBase {
    id: string
    label: string
    plannedDuration: Temporal.Duration
}

export const defineInteractionScene = <ObjectIds extends string>(d: SceneDefinitionBase & {
    type: "interaction"
    objects: IdMap<SceneObjectDefinition & { id: ObjectIds }>
    interactions: Array<UseInteractionDefinition<ObjectIds> | CombineInteractionDefinition<ObjectIds>>
}) => ({
    ...d,
    interactionsById: new Map(d.interactions.map(i => [getSuggestedInteractionId(getSuggestedInteractionFromDefinition(i)), i]))
})

export interface InteractionSceneDefinition<ObjectIds extends string = string> extends SceneDefinitionBase {
    type: "interaction"
    objects: IdMap<SceneObjectDefinition & { id: ObjectIds }>
    interactionsById: Map<string, UseInteractionDefinition<ObjectIds> | CombineInteractionDefinition<ObjectIds>>
}

export interface SceneObjectDefinition {
    id: string
    label: string
    reveal: boolean
    relevant: boolean
    completionSteps?: number
}

export interface UseInteractionDefinition<AvailableObjectIds extends string> {
    type: "use"
    objectId: AvailableObjectIds
    consume: boolean
    note: string
}

export interface CombineInteractionDefinition<AvailableObjectIds extends string> {
    type: "combine"
    inputObjects: Map<AvailableObjectIds, CombinationInteractionDefinitionInputOptions>
    outputObjectIds: Set<AvailableObjectIds>
    note: string
}

export interface CombinationInteractionDefinitionInputOptions {
    consume: boolean
}

export interface ChoiceSceneDefinition extends SceneDefinitionBase {
    type: "choice"
    options: ChoiceSceneDefinitionOption[]
}

export interface ChoiceSceneDefinitionOption {
    id: string
    label: string
}

export interface TextSceneDefinition extends SceneDefinitionBase {
    type: "text"
    text: string
}

export type SceneDefinition = InteractionSceneDefinition | ChoiceSceneDefinition | TextSceneDefinition
